#! /bin/sh

unset LFS
source lfs.lib

check_root

pass0_warning

essential_file()
{
    touch /var/run/utmp /var/log/{btmp,lastlog,wtmp}
    chgrp -v utmp /var/run/utmp /var/log/lastlog
    chmod -v 664 /var/run/utmp /var/log/lastlog
}

build_linux_api_headers()
{
    #检测
    cd /sources

    check_file linux-2.6.22.5.tar.bz2
    check_dir linux-2.6.22.5

    #解压
    tar jxvf linux-2.6.22.5.tar.bz2
    cd linux-2.6.22.5

    #特殊
    sed -i '/scsi/d' include/Kbuild

    before_date=`date -R`

    #编译
    make mrproper
    make headers_check
    make INSTALL_HDR_PATH=dest headers_install
    cp -rv dest/include/* /usr/include

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R linux-2.6.22.5
}

build_man_pages()
{
    #检测
    cd /sources

    check_file man-pages-2.63.tar.bz2
    check_dir man-pages-2.63

    #解压
    tar jxvf man-pages-2.63.tar.bz2
    cd man-pages-2.63

    before_date=`date -R`

    #编译
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R man-pages-2.63
}

build_glibc()
{
    #检测
    cd /sources

    check_file glibc-2.5.1.tar.bz2
    check_file glibc-libidn-2.5.1.tar.gz
    check_dir glibc-2.5.1
    check_dir glibc-build

    #解压
    tar jxvf glibc-2.5.1.tar.bz2
    cd glibc-2.5.1
    tar zxvf ../glibc-libidn-2.5.1.tar.gz
    mv glibc-libidn-2.5.1 libidn

    #特殊
    sed -i '/vi_VN.TCVN/d' localedata/SUPPORTED
    sed -i \
	's|libs -o|libs -L/usr/lib -Wl,-dynamic-linker=/lib/ld-linux.so.2 -o|' \
        scripts/test-installation.pl
    sed -i 's|@BASH@|/bin/bash|' elf/ldd.bash.in

    before_date=`date -R`

    #编译
    mkdir -v ../glibc-build
    cd ../glibc-build

    ../glibc-2.5.1/configure --prefix=/usr \
	--disable-profile --enable-add-ons \
	--enable-kernel=2.6.0 --libexecdir=/usr/lib/glibc
    make || eval "echo \"error: make error in glibc\" && exit 1"
#    make -k check 2>&1 | tee glibc-check-log
#    grep Error glibc-check-log
#    echo "glibc check done, please press enter"
#    read input

    touch /etc/ld.so.conf
    make install

    #国际化，这里全部安装
    make localedata/install-locales

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #配置
cat > /etc/nsswitch.conf << "EOF"
# Begin /etc/nsswitch.conf

passwd: files
group: files
shadow: files

hosts: files dns
networks: files

protocols: files
services: files
ethers: files
rpc: files

# End /etc/nsswitch.conf
EOF

cp -v --remove-destination /usr/share/zoneinfo/Asia/Shanghai \
    /etc/localtime

cat > /etc/ld.so.conf << "EOF"
# Begin /etc/ld.so.conf

/usr/local/lib
/opt/lib

# End /etc/ld.so.conf
EOF

    #清理
    cd /sources/
    rm -f -R glibc-2.5.1
    rm -f -R glibc-build
}

adjust_toolchain()
{
    mv -v /tools/bin/{ld,ld-old}
    mv -v /tools/$(gcc -dumpmachine)/bin/{ld,ld-old}
    mv -v /tools/bin/{ld-new,ld}
    ln -sv /tools/bin/ld /tools/$(gcc -dumpmachine)/bin/ld

    gcc -dumpspecs | sed \
	-e 's@/tools/lib/ld-linux.so.2@/lib/ld-linux.so.2@g' \
	-e '/\*startfile_prefix_spec:/{n;s@.*@/usr/lib/ @}' \
	-e '/\*cpp:/{n;s@$@ -isystem /usr/include@}' > \
	`dirname $(gcc --print-libgcc-file-name)`/specs

    echo 'main(){}' > dummy.c
    cc dummy.c -v -Wl,--verbose &> dummy.log
    readelf -l a.out | grep ': /lib'

    grep -o '/usr/lib.*/crt[1in].*succeeded' dummy.log
    grep -B1 '^ /usr/include' dummy.log
    grep 'SEARCH.*/usr/lib' dummy.log |sed 's|; |\n|g'
    grep "/lib/libc.so.6 " dummy.log
    grep found dummy.log

#read input

    rm -v dummy.c a.out
}

build_binutils()
{
    #检测
    cd /sources

    check_file binutils-2.17.tar.bz2
    check_dir binutils-2.17
    check_dir binutils-build

    #测试
    expect -c "spawn ls"
#    read input

    #解压
    tar jxvf binutils-2.17.tar.bz2
    cd binutils-2.17

    before_date=`date -R`

    #编译
    mkdir -v ../binutils-build
    cd ../binutils-build
    ../binutils-2.17/configure --prefix=/usr \
	--enable-shared
    make tooldir=/usr || eval "echo \"error: make error in binutils\" && exit 1"
#    make check
    make tooldir=/usr install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    cp -v ../binutils-2.17/include/libiberty.h /usr/include

    #清理
    cd /sources/
    rm -f -R binutils-2.17
    rm -f -R binutils-build
}

build_gcc()
{
    #检测
    cd /sources

    check_file gcc-4.1.2.tar.bz2
    check_dir gcc-4.1.2
    check_dir gcc-build

    #解压
    tar jxvf gcc-4.1.2.tar.bz2
    cd gcc-4.1.2

    #特殊
    sed -i 's/install_to_$(INSTALL_DEST) //' libiberty/Makefile.in
    sed -i 's/^XCFLAGS =$/& -fomit-frame-pointer/' gcc/Makefile.in
    sed -i 's@\./fixinc\.sh@-c true@' gcc/Makefile.in
    sed -i 's/@have_mktemp_command@/yes/' gcc/gccbug.in

    before_date=`date -R`

    #编译
    mkdir -v ../gcc-build
    cd ../gcc-build
    ../gcc-4.1.2/configure --prefix=/usr \
	--libexecdir=/usr/lib --enable-shared \
	--enable-threads=posix --enable-__cxa_atexit \
	--enable-clocale=gnu --enable-languages=c,c++
    make || eval "echo \"error: make error in gcc\" && exit 1"
#    make -k check
#    ../gcc-4.1.2/contrib/test_summary
#    echo "check done, please press enter to continue"
#    read input
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    ln -sv ../usr/bin/cpp /lib
    ln -sv gcc /usr/bin/cc

    #测试
    echo 'main(){}' > dummy.c
    cc dummy.c -v -Wl,--verbose &> dummy.log
    readelf -l a.out | grep ': /lib'
    grep -o '/usr/lib.*/crt[1in].*succeeded' dummy.log
    grep -B3 '^ /usr/include' dummy.log
    grep 'SEARCH.*/usr/lib' dummy.log |sed 's|; |\n|g'
    grep "/lib/libc.so.6 " dummy.log
    grep found dummy.log
#    echo "check log done, please press enter to continue"
#    read input
    rm -f -v dummy.c a.out dummy.log

    #清理
    cd /sources/
    rm -f -R gcc-4.1.2
    rm -f -R gcc-build
}

build_db()
{
    #检测
    cd /sources

    check_file db-4.5.20.tar.gz
    check_file db-4.5.20-fixes-1.patch
    check_dir db-4.5.20

    #解压
    tar zxvf db-4.5.20.tar.gz
    cd db-4.5.20

    #打补丁
    patch -Np1 -i ../db-4.5.20-fixes-1.patch

    before_date=`date -R`

    #编译
    cd build_unix
    ../dist/configure --prefix=/usr --enable-compat185 --enable-cxx
    make || eval "echo \"error: make error in db\" && exit 1"
    make docdir=/usr/share/doc/db-4.5.20 install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    chown -Rv root:root /usr/share/doc/db-4.5.20

    #清理
    cd /sources/
    rm -f -R db-4.5.20
}

build_sed()
{
    #检测
    cd /sources

    check_file sed-4.1.5.tar.gz
    check_dir sed-4.1.5

    #解压
    tar zxvf sed-4.1.5.tar.gz
    cd sed-4.1.5

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr --bindir=/bin --enable-html
    make || eval "echo \"error: make error in sed\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R sed-4.1.5
}

build_e2fsprogs()
{
    #检测
    cd /sources

    check_file e2fsprogs-1.40.2.tar.gz
    check_dir e2fsprogs-1.40.2

    #解压
    tar zxvf e2fsprogs-1.40.2.tar.gz
    cd e2fsprogs-1.40.2

    #特殊
    sed -i -e 's@/bin/rm@/tools&@' lib/blkid/test_probe.in

    before_date=`date -R`

    #编译
    mkdir -v build
    cd build
    ../configure --prefix=/usr --with-root-prefix="" \
	--enable-elf-shlibs
    make || eval "echo \"error: make error in e2fsprogs\" && exit 1"
    make install
    make install-libs

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R e2fsprogs-1.40.2
}

build_coreutils()
{
    #检测
    cd /sources

    check_file coreutils-6.9.tar.bz2
    check_file coreutils-6.9-uname-1.patch
    check_file coreutils-6.9-suppress_uptime_kill_su-1.patch
    check_file coreutils-6.9-i18n-1.patch
    check_dir coreutils-6.9

    #解压
    tar jxvf coreutils-6.9.tar.bz2
    cd coreutils-6.9

    before_date=`date -R`

    #打补丁
    patch -Np1 -i ../coreutils-6.9-uname-1.patch
    patch -Np1 -i ../coreutils-6.9-suppress_uptime_kill_su-1.patch
    patch -Np1 -i ../coreutils-6.9-i18n-1.patch

    #特殊
    chmod +x tests/sort/sort-mb-tests

    #编译
    ./configure --prefix=/usr
    make || eval "echo \"error: make error in coreutils\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    mv -v /usr/bin/{cat,chgrp,chmod,chown,cp,date,dd,df,echo} /bin
    mv -v /usr/bin/{false,hostname,ln,ls,mkdir,mknod,mv,pwd,readlink,rm} /bin
    mv -v /usr/bin/{rmdir,stty,sync,true,uname} /bin
    mv -v /usr/bin/chroot /usr/sbin
    mv -v /usr/bin/{head,sleep,nice} /bin

    #清理
    cd /sources/
    rm -f -R coreutils-6.9
}

build_iana_etc()
{
    #检测
    cd /sources

    check_file iana-etc-2.20.tar.bz2
    check_dir iana-etc-2.20

    #解压
    tar jxvf iana-etc-2.20.tar.bz2
    cd iana-etc-2.20

    before_date=`date -R`

    #编译
    make || eval "echo \"error: make error in iana-etc\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R iana-etc-2.20
}

build_m4()
{
    #检测
    cd /sources

    check_file m4-1.4.10.tar.bz2
    check_dir m4-1.4.10

    #解压    
    tar jxvf m4-1.4.10.tar.bz2
    cd m4-1.4.10

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr
    make || eval "echo \"error: make error in m4\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R m4-1.4.10
}

build_bison()
{
    #检测
    cd /sources

    check_file bison-2.3.tar.bz2
    check_dir bison-2.3

    #解压    
    tar jxvf bison-2.3.tar.bz2
    cd bison-2.3

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr
    echo '#define YYENABLE_NLS 1' >> config.h
    make || eval "echo \"error: make error in bison\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R bison-2.3
}

build_ncurses()
{
    #检测
    cd /sources

    check_file ncurses-5.6.tar.gz
    check_file ncurses-5.6-coverity_fixes-1.patch
    check_dir ncurses-5.6

    #打补丁
    patch -Np1 -i ../ncurses-5.6-coverity_fixes-1.patch

    #解压
    tar zxvf ncurses-5.6.tar.gz
    cd ncurses-5.6

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr --with-shared --without-debug --enable-widec
    make || eval "echo \"error: make error in ncurses\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    chmod -v 644 /usr/lib/libncurses++w.a
    mv -v /usr/lib/libncursesw.so.5* /lib
    ln -sfv ../../lib/libncursesw.so.5 /usr/lib/libncursesw.so
    for lib in curses ncurses form panel menu ; do \
	rm -vf /usr/lib/lib${lib}.so ; \
	echo "INPUT(-l${lib}w)" >/usr/lib/lib${lib}.so ; \
	ln -sfv lib${lib}w.a /usr/lib/lib${lib}.a ; \
	done
    ln -sfv libncurses++w.a /usr/lib/libncurses++.a
    rm -vf /usr/lib/libcursesw.so
    echo "INPUT(-lncursesw)" >/usr/lib/libcursesw.so
    ln -sfv libncurses.so /usr/lib/libcurses.so
    ln -sfv libncursesw.a /usr/lib/libcursesw.a
    ln -sfv libncurses.a /usr/lib/libcurses.a

    #清理
    cd /sources/
    rm -f -R ncurses-5.6
}

build_procps()
{
    #检测
    cd /sources

    check_file procps-3.2.7.tar.gz
    check_dir procps-3.2.7

    #解压    
    tar zxvf procps-3.2.7.tar.gz
    cd procps-3.2.7

    before_date=`date -R`

    #编译
    make || eval "echo \"error: make error in procps\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R procps-3.2.7
}

build_libtool()
{
    #检测
    cd /sources

    check_file libtool-1.5.24.tar.gz
    check_dir libtool-1.5.24

    #解压    
    tar zxvf libtool-1.5.24.tar.gz
    cd libtool-1.5.24

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr
    make || eval "echo \"error: make error in libtool\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R libtool-1.5.24
}

build_perl()
{
    #检测
    cd /sources

    check_file perl-5.8.8.tar.bz2
    check_file perl-5.8.8-libc-2.patch
    check_dir perl-5.8.8

    #特殊
    echo "127.0.0.1 localhost $(hostname)" > /etc/hosts

    #解压
    tar jxvf perl-5.8.8.tar.bz2
    cd perl-5.8.8

    before_date=`date -R`

    #编译
    ./configure.gnu --prefix=/usr \
	-Dman1dir=/usr/share/man/man1 \
	-Dman3dir=/usr/share/man/man3 \
	-Dpager="/usr/bin/less -isR"
    make || eval "echo \"error: make error in perl\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R perl-5.8.8
}

build_readline()
{
    #检测
    cd /sources

    check_file readline-5.2.tar.gz
    check_file readline-5.2-fixes-3.patch
    check_dir readline-5.2

    #解压
    tar zxvf readline-5.2.tar.gz
    cd readline-5.2

    #特殊
    sed -i '/MV.*old/d' Makefile.in
    sed -i '/{OLDSUFF}/c:' support/shlib-install

    #打补丁
    patch -Np1 -i ../readline-5.2-fixes-3.patch

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr --libdir=/lib
    make SHLIB_LIBS=-lncurses || eval "echo \"error: make error in readline\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    mv -v /lib/lib{readline,history}.a /usr/lib
    rm -v /lib/lib{readline,history}.so
    ln -sfv ../../lib/libreadline.so.5 /usr/lib/libreadline.so
    ln -sfv ../../lib/libhistory.so.5 /usr/lib/libhistory.so

    #清理
    cd /sources/
    rm -f -R readline-5.2
}

build_zlib()
{
    #检测
    cd /sources

    check_file zlib-1.2.3.tar.gz
    check_dir zlib-1.2.3

    #解压
    tar zxvf zlib-1.2.3.tar.gz
    cd zlib-1.2.3

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr --shared --libdir=/lib
    make || eval "echo \"error: make error in zlib\" && exit 1"
    make install
    rm -v /lib/libz.so
    ln -sfv ../../lib/libz.so.1.2.3 /usr/lib/libz.so
    make clean
    ./configure --prefix=/usr
    make || eval "echo \"error: make error in zlib\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    chmod -v 644 /usr/lib/libz.a

    #清理
    cd /sources/
    rm -f -R zlib-1.2.3
}

build_autoconf()
{
    #检测
    cd /sources

    check_file autoconf-2.61.tar.bz2
    check_dir autoconf-2.61

    #解压
    tar jxvf autoconf-2.61.tar.bz2
    cd autoconf-2.61

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr
    make || eval "echo \"error: make error in autoconf\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R autoconf-2.61
}

build_automake()
{
    #检测
    cd /sources

    check_file automake-1.10.tar.bz2
    check_dir automake-1.10

    #解压
    tar jxvf automake-1.10.tar.bz2
    cd automake-1.10

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr
    make || eval "echo \"error: make error in automake\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R automake-1.10
}

bash_comment()
{
    echo "****************"
    echo "please run the following commands"
    echo
    echo "# exec /bin/bash --login +h"
    echo
    echo "****************"
}

build_bash()
{
    #检测
    cd /sources

    check_file bash-3.2.tar.gz
    check_file bash-doc-3.2.tar.gz
    check_file bash-3.2-fixes-5.patch
    check_dir bash-3.2

    #解压
    tar zxvf bash-3.2.tar.gz
    cd bash-3.2
    tar zxvf ../bash-doc-3.2.tar.gz

    #特殊
    sed -i "s|htmldir = @htmldir@|htmldir = /usr/share/doc/bash-3.2|" \
	Makefile.in

    #打补丁
    patch -Np1 -i ../bash-3.2-fixes-5.patch

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr --bindir=/bin \
	--without-bash-malloc --with-installed-readline
    make || eval "echo \"error: make error in bash\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    ln -vs bash /tools/bin/sh

    #清理
    cd /sources/
    rm -f -R bash-3.2

    #重新登录
    bash_comment
}

before_date_sum=`date -R`

essential_file
build_linux_api_headers
build_man_pages
build_glibc
adjust_toolchain
build_binutils
build_gcc
build_db
build_sed
build_e2fsprogs
build_coreutils
build_iana_etc
build_m4
build_bison
build_ncurses
build_procps
build_libtool
build_perl
build_readline
build_zlib
build_autoconf
build_automake
build_bash

#计时
after_date_sum=`date -R`
echo "$before_date_sum"
echo "$after_date_sum"
