#! /bin/sh

unset LFS
source lfs.lib

check_root

pass0_warning

build_bzip2()
{
    #检测
    cd /sources

    check_file bzip2-1.0.4.tar.gz
    check_file bzip2-1.0.4-install_docs-1.patch
    check_dir bzip2-1.0.4

    #解压
    tar zxvf bzip2-1.0.4.tar.gz
    cd bzip2-1.0.4

    #打补丁
    patch -Np1 -i ../bzip2-1.0.4-install_docs-1.patch

    before_date=`date -R`

    #编译
    make -f Makefile-libbz2_so
    make clean
    make || eval "echo \"error: make error in bzip2\" && exit 1"
    make PREFIX=/usr install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    cp -v bzip2-shared /bin/bzip2
    cp -av libbz2.so* /lib
    ln -sv ../../lib/libbz2.so.1.0 /usr/lib/libbz2.so
    rm -v /usr/bin/{bunzip2,bzcat,bzip2}
    ln -sv bzip2 /bin/bunzip2
    ln -sv bzip2 /bin/bzcat

    #清理
    cd /sources/
    rm -f -R bzip2-1.0.4
}

build_diffutils()
{
    #检测
    cd /sources

    check_file diffutils-2.8.1.tar.gz
    check_file diffutils-2.8.1-i18n-1.patch
    check_dir diffutils-2.8.1

    #解压
    tar zxvf diffutils-2.8.1.tar.gz
    cd diffutils-2.8.1

    #打补丁
    patch -Np1 -i ../diffutils-2.8.1-i18n-1.patch

    #特殊
    touch man/diff.1

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr
    make || eval "echo \"error: make error in diffutils\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R diffutils-2.8.1
}

build_file()
{
    #检测
    cd /sources

    check_file file-4.21.tar.gz
    check_dir file-4.21

    #解压
    tar zxvf file-4.21.tar.gz
    cd file-4.21

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr
    make || eval "echo \"error: make error in file\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R file-4.21
}

build_findutils()
{
    #检测
    cd /sources

    check_file findutils-4.2.31.tar.gz
    check_dir findutils-4.2.31

    #解压
    tar zxvf findutils-4.2.31.tar.gz
    cd findutils-4.2.31

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr --libexecdir=/usr/lib/findutils \
	--localstatedir=/var/lib/locate
    make || eval "echo \"error: make error in findutils\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    mv -v /usr/bin/find /bin
    sed -i -e 's/find:=${BINDIR}/find:=\/bin/' /usr/bin/updatedb

    #清理
    cd /sources/
    rm -f -R findutils-4.2.31
}

build_flex()
{
    #检测
    cd /sources

    check_file flex-2.5.33.tar.bz2
    check_dir flex-2.5.33

    #解压
    tar jxvf flex-2.5.33.tar.bz2
    cd flex-2.5.33

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr
    make || eval "echo \"error: make error in flex\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    ln -sv libfl.a /usr/lib/libl.a
    cat > /usr/bin/lex << "EOF"
#!/bin/sh
# Begin /usr/bin/lex

exec /usr/bin/flex -l "$@"

# End /usr/bin/lex
EOF
    chmod -v 755 /usr/bin/lex

    #清理
    cd /sources/
    rm -f -R flex-2.5.33
}

build_grub()
{
    #检测
    cd /sources

    check_file grub-0.97.tar.gz
    check_file grub-0.97-disk_geometry-1.patch
    check_dir grub-0.97

    #解压
    tar zxvf grub-0.97.tar.gz
    cd grub-0.97

    #打补丁
    patch -Np1 -i ../grub-0.97-disk_geometry-1.patch

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr
    make || eval "echo \"error: make error in grub\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊，/boot有可能是在另外一个挂载
    mkdir -v /boot/grub
    cp -v /usr/lib/grub/i386-pc/stage{1,2} /boot/grub
    cp -v /usr/lib/grub/i386-pc/e2fs_stage1_5 /boot/grub

    #清理
    cd /sources/
    rm -f -R grub-0.97
}

build_gawk()
{
    #检测
    cd /sources

    check_file gawk-3.1.5.tar.bz2
    check_file gawk-3.1.5-segfault_fix-1.patch
    check_dir gawk-3.1.5

    #解压
    tar jxvf gawk-3.1.5.tar.bz2
    cd gawk-3.1.5

    #打补丁
    patch -Np1 -i ../gawk-3.1.5-segfault_fix-1.patch

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr --libexecdir=/usr/lib
    cat >> config.h << "EOF"
#define HAVE_LANGINFO_CODESET 1
#define HAVE_LC_MESSAGES 1
EOF
    make || eval "echo \"error: make error in gawk\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R gawk-3.1.5
}

build_gettext()
{
    #检测
    cd /sources

    check_file gettext-0.16.1.tar.gz
    check_dir gettext-0.16.1

    #解压
    tar zxvf gettext-0.16.1.tar.gz
    cd gettext-0.16.1

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr
    make || eval "echo \"error: make error in gettext\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R gettext-0.16.1
}

build_grep()
{
    #检测
    cd /sources

    check_file grep-2.5.1a.tar.bz2
    check_file grep-2.5.1a-redhat_fixes-2.patch
    check_dir grep-2.5.1a

    #解压
    tar jxvf grep-2.5.1a.tar.bz2
    cd grep-2.5.1a

    #打补丁
    patch -Np1 -i ../grep-2.5.1a-redhat_fixes-2.patch

    #特殊
    chmod +x tests/fmbtest.sh

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr --bindir=/bin
    make || eval "echo \"error: make error in grep\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R grep-2.5.1a
}

build_groff()
{
    #检测
    cd /sources

    check_file groff-1.18.1.4.tar.gz
    check_file groff-1.18.1.4-debian_fixes-1.patch
    check_dir groff-1.18.1.4

    #解压
    tar zxvf groff-1.18.1.4.tar.gz
    cd groff-1.18.1.4

    #打补丁
    patch -Np1 -i ../groff-1.18.1.4-debian_fixes-1.patch

    #特殊
    sed -i -e 's/2010/002D/' -e 's/2212/002D/' \
	-e 's/2018/0060/' -e 's/2019/0027/' font/devutf8/R.proto

    before_date=`date -R`

    #编译
    PAGE=A4 ./configure --prefix=/usr --enable-multibyte
    make || eval "echo \"error: make error in groff\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    ln -sv eqn /usr/bin/geqn
    ln -sv tbl /usr/bin/gtbl

    #清理
    cd /sources/
    rm -f -R groff-1.18.1.4
}

build_gzip()
{
    #检测
    cd /sources

    check_file gzip-1.3.12.tar.gz
    check_dir gzip-1.3.12

    #解压
    tar zxvf gzip-1.3.12.tar.gz
    cd gzip-1.3.12

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr --bindir=/bin
    make || eval "echo \"error: make error in gzip\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    mv -v /bin/{gzexe,uncompress,zcmp,zdiff,zegrep} /usr/bin
    mv -v /bin/{zfgrep,zforce,zgrep,zless,zmore,znew} /usr/bin

    #清理
    cd /sources/
    rm -f -R gzip-1.3.12
}

build_inetutils()
{
    #检测
    cd /sources

    check_file inetutils-1.5.tar.gz
    check_file inetutils-1.5-no_server_man_pages-2.patch
    check_dir inetutils-1.5

    #解压
    tar zxvf inetutils-1.5.tar.gz
    cd inetutils-1.5

    #打补丁
    patch -Np1 -i ../inetutils-1.5-no_server_man_pages-2.patch

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr --libexecdir=/usr/sbin \
	--sysconfdir=/etc --localstatedir=/var \
	--disable-ifconfig --disable-logger --disable-syslogd \
	--disable-whois --disable-servers
    make || eval "echo \"error: make error in inetutils\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    mv -v /usr/bin/ping /bin

    #清理
    cd /sources/
    rm -f -R inetutils-1.5
}

build_iproute()
{
    #检测
    cd /sources

    check_file iproute2-2.6.20-070313.tar.gz
    check_dir iproute-2.6.20-070313

    #解压
    tar zxvf iproute2-2.6.20-070313.tar.gz
    cd iproute-2.6.20-070313

    #特殊
    sed -i -e '/tc-bfifo.8/d' -e '/tc-pfifo.8/s/pbfifo/bfifo/' Makefile

    before_date=`date -R`

    #编译
    make SBINDIR=/sbin || eval "echo \"error: make error in iproute\" && exit 1"
    make SBINDIR=/sbin install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    mv -v /sbin/arpd /usr/sbin

    #清理
    cd /sources/
    rm -f -R iproute-2.6.20-070313
}

build_kbd()
{
    #检测
    cd /sources

    check_file kbd-1.12.tar.bz2
    check_file kbd-1.12-backspace-1.patch
    check_file kbd-1.12-gcc4_fixes-1.patch
    check_dir kbd-1.12

    #解压
    tar jxvf kbd-1.12.tar.bz2
    cd kbd-1.12

    #打补丁
    patch -Np1 -i ../kbd-1.12-backspace-1.patch
    patch -Np1 -i ../kbd-1.12-gcc4_fixes-1.patch

    before_date=`date -R`

    #编译
    ./configure --datadir=/lib/kbd
    make || eval "echo \"error: make error in kbd\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    mv -v /usr/bin/{kbd_mode,openvt,setfont} /bin

    #清理
    cd /sources/
    rm -f -R kbd-1.12
}

build_less()
{
    #检测
    cd /sources

    check_file less-406.tar.gz
    check_dir less-406

    #解压
    tar zxvf less-406.tar.gz
    cd less-406

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr --sysconfdir=/etc
    make || eval "echo \"error: make error in less\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R less-406
}

build_make()
{
    #检测
    cd /sources

    check_file make-3.81.tar.bz2
    check_dir make-3.81

    #解压
    tar jxvf make-3.81.tar.bz2
    cd make-3.81

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr
    make || eval "echo \"error: make error in make\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R make-3.81
}

build_man_db()
{
    #检测
    cd /sources

    check_file man-db-2.4.4.tar.gz
    check_file man-db-2.4.4-fixes-1.patch
    check_dir man-db-2.4.4

    #解压
    tar zxvf man-db-2.4.4.tar.gz
    cd man-db-2.4.4

    #特殊
    mv man/de{_DE.88591,}
    mv man/es{_ES.88591,}
    mv man/it{_IT.88591,}
    mv man/ja{_JP.eucJP,}
    sed -i 's,\*_\*,??,' man/Makefile.in
    sed -i -e '\%\t/usr/man%d' -e '\%\t/usr/local/man%d' src/man_db.conf.in

    cat >> include/manconfig.h.in << "EOF"
#define WEB_BROWSER "exec /usr/bin/lynx"
#define COL "/usr/bin/col"
#define VGRIND "/usr/bin/vgrind"
#define GRAP "/usr/bin/grap"
EOF

    #打补丁
    patch -Np1 -i ../man-db-2.4.4-fixes-1.patch

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr --enable-mb-groff --disable-setuid
    make || eval "echo \"error: make error in man_db\" && exit 1"
    make install

    #特殊
    cat >> convert-mans << "EOF"
#!/bin/sh -e
FROM="$1"
TO="$2"
shift ; shift
while [ $# -gt 0 ]
do
        FILE="$1"
        shift
        iconv -f "$FROM" -t "$TO" "$FILE" >.tmp.iconv
        mv .tmp.iconv "$FILE"
done
EOF
    install -m755 convert-mans  /usr/bin

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R man-db-2.4.4
}

build_mktemp()
{
    #检测
    cd /sources

    check_file mktemp-1.5.tar.gz
    check_file mktemp-1.5-add_tempfile-3.patch
    check_dir mktemp-1.5

    #解压
    tar zxvf mktemp-1.5.tar.gz
    cd mktemp-1.5

    #打补丁
    patch -Np1 -i ../mktemp-1.5-add_tempfile-3.patch

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr --with-libc
    make || eval "echo \"error: make error in mktemp\" && exit 1"
    make install
    make install-tempfile

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R mktemp-1.5
}

build_module_init_tools()
{
    #检测
    cd /sources

    check_file module-init-tools-3.2.2.tar.bz2
    check_file module-init-tools-3.2.2-modprobe-1.patch
    check_dir module-init-tools-3.2.2

    #解压
    tar jxvf module-init-tools-3.2.2.tar.bz2
    cd module-init-tools-3.2.2

    #打补丁
    patch -Np1 -i ../module-init-tools-3.2.2-modprobe-1.patch

    before_date=`date -R`

    #编译
    ./configure --prefix=/ --enable-zlib
    make || eval "echo \"error: make error in module_init_tools\" && exit 1"
    make INSTALL=install install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R module-init-tools-3.2.2
}

build_patch()
{
    #检测
    cd /sources

    check_file patch-2.5.4.tar.gz
    check_dir patch-2.5.4

    #解压
    tar zxvf patch-2.5.4.tar.gz
    cd patch-2.5.4

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr
    make || eval "echo \"error: make error in patch\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R patch-2.5.4
}

build_psmisc()
{
    #检测
    cd /sources

    check_file psmisc-22.5.tar.gz
    check_dir psmisc-22.5

    #解压
    tar zxvf psmisc-22.5.tar.gz
    cd psmisc-22.5

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr --exec-prefix=""
    make || eval "echo \"error: make error in psmisc\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    mv -v /bin/pstree* /usr/bin
    ln -sv killall /bin/pidof

    #清理
    cd /sources/
    rm -f -R psmisc-22.5
}

build_shadow()
{
    #检测
    cd /sources

    check_file shadow-4.0.18.1.tar.bz2
    check_file shadow-4.0.18.1-useradd_fix-2.patch
    check_dir shadow-4.0.18.1

    #解压
    tar jxvf shadow-4.0.18.1.tar.bz2
    cd shadow-4.0.18.1

    #打补丁
    patch -Np1 -i ../shadow-4.0.18.1-useradd_fix-2.patch

    before_date=`date -R`

    #编译
    ./configure --libdir=/lib --sysconfdir=/etc --enable-shared \
	--without-selinux
    sed -i 's/groups$(EXEEXT) //' src/Makefile
    find man -name Makefile -exec sed -i 's/groups\.1 / /' {} \;
    sed -i -e 's/ ko//' -e 's/ zh_CN zh_TW//' man/Makefile
    for i in de es fi fr id it pt_BR; do
	convert-mans UTF-8 ISO-8859-1 man/${i}/*.?
    done
    for i in cs hu pl; do
	convert-mans UTF-8 ISO-8859-2 man/${i}/*.?
    done
    convert-mans UTF-8 EUC-JP man/ja/*.?
    convert-mans UTF-8 KOI8-R man/ru/*.?
    convert-mans UTF-8 ISO-8859-9 man/tr/*.?
    sed -i -e 's@#MD5_CRYPT_ENAB.no@MD5_CRYPT_ENAB yes@' \
	-e 's@/var/spool/mail@/var/mail@' etc/login.defs
    make || eval "echo \"error: make error in shadow\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    mv -v /usr/bin/passwd /bin
    mv -v /lib/libshadow.*a /usr/lib
    rm -v /lib/libshadow.so
    ln -sfv ../../lib/libshadow.so.0 /usr/lib/libshadow.so

    pwconv
    grpconv
    useradd -D -b /home
    sed -i 's/yes/no/' /etc/default/useradd

#optional
#    echo "enter root's password"
#    passwd root

    #清理
    cd /sources/
    rm -f -R shadow-4.0.18.1
}

passwd_comment()
{
    echo "******** warning ********"
    echo "remeber to run the following command!"
    echo
    echo "# passwd root"
    echo
    echo "******** warning ********"
}

build_sysklogd()
{
    #检测
    cd /sources

    check_file sysklogd-1.4.1.tar.gz
    check_file sysklogd-1.4.1-fixes-2.patch
    check_file sysklogd-1.4.1-8bit-1.patch
    check_dir sysklogd-1.4.1

    #解压
    tar zxvf sysklogd-1.4.1.tar.gz
    cd sysklogd-1.4.1

    #打补丁
    patch -Np1 -i ../sysklogd-1.4.1-fixes-2.patch
    patch -Np1 -i ../sysklogd-1.4.1-8bit-1.patch

    before_date=`date -R`

    #编译
    make || eval "echo \"error: make error in sysklogd\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    cat > /etc/syslog.conf << "EOF"
# Begin /etc/syslog.conf

auth,authpriv.* -/var/log/auth.log
*.*;auth,authpriv.none -/var/log/sys.log
daemon.* -/var/log/daemon.log
kern.* -/var/log/kern.log
mail.* -/var/log/mail.log
user.* -/var/log/user.log
*.emerg *

# End /etc/syslog.conf
EOF

    #清理
    cd /sources/
    rm -f -R sysklogd-1.4.1
}

build_sysvinit()
{
    #检测
    cd /sources

    check_file sysvinit-2.86.tar.gz
    check_dir sysvinit-2.86

    #解压
    tar zxvf sysvinit-2.86.tar.gz
    cd sysvinit-2.86

    #特殊
    sed -i 's@Sending processes@& configured via /etc/inittab@g' \
	src/init.c

    before_date=`date -R`

    #编译
    make -C src || eval "echo \"error: make error in sysvinit\" && exit 1"
    make -C src install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    cat > /etc/inittab << "EOF"
# Begin /etc/inittab

id:3:initdefault:

si::sysinit:/etc/rc.d/init.d/rc sysinit

l0:0:wait:/etc/rc.d/init.d/rc 0
l1:S1:wait:/etc/rc.d/init.d/rc 1
l2:2:wait:/etc/rc.d/init.d/rc 2
l3:3:wait:/etc/rc.d/init.d/rc 3
l4:4:wait:/etc/rc.d/init.d/rc 4
l5:5:wait:/etc/rc.d/init.d/rc 5
l6:6:wait:/etc/rc.d/init.d/rc 6

ca:12345:ctrlaltdel:/sbin/shutdown -t1 -a -r now

su:S016:once:/sbin/sulogin

1:2345:respawn:/sbin/agetty tty1 9600
2:2345:respawn:/sbin/agetty tty2 9600
3:2345:respawn:/sbin/agetty tty3 9600
4:2345:respawn:/sbin/agetty tty4 9600
5:2345:respawn:/sbin/agetty tty5 9600
6:2345:respawn:/sbin/agetty tty6 9600

# End /etc/inittab
EOF

    #清理
    cd /sources/
    rm -f -R sysvinit-2.86
}

build_tar()
{
    #检测
    cd /sources

    check_file tar-1.18.tar.bz2
    check_dir tar-1.18

    #解压
    tar jxvf tar-1.18.tar.bz2
    cd tar-1.18

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr --bindir=/bin --libexecdir=/usr/sbin
    make || eval "echo \"error: make error in tar\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R tar-1.18
}

build_texinfo()
{
    #检测
    cd /sources

    check_file texinfo-4.9.tar.bz2
    check_file texinfo-4.9-multibyte-1.patch
    check_file texinfo-4.9-tempfile_fix-1.patch
    check_dir texinfo-4.9

    #解压
    tar jxvf texinfo-4.9.tar.bz2
    cd texinfo-4.9

    #打补丁
    patch -Np1 -i ../texinfo-4.9-multibyte-1.patch
    patch -Np1 -i ../texinfo-4.9-tempfile_fix-1.patch

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr
    make || eval "echo \"error: make error in texinfo\" && exit 1"
    make install
    make TEXMF=/usr/share/texmf install-tex

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    cd /usr/share/info
    rm dir
    for f in *
    do
	install-info $f dir 2>/dev/null
    done

    #清理
    cd /sources/
    rm -f -R texinfo-4.9
}

build_udev()
{
    #检测
    cd /sources

    check_file udev-113.tar.bz2
    check_file udev-config-6.3.tar.bz2
    check_dir texinfo-4.9

    #解压
    tar jxvf udev-113.tar.bz2
    cd udev-113

    #特殊
    install -dv /lib/{firmware,udev/devices/{pts,shm}}
    mknod -m0666 /lib/udev/devices/null c 1 3
    ln -sv /proc/self/fd /lib/udev/devices/fd
    ln -sv /proc/self/fd/0 /lib/udev/devices/stdin
    ln -sv /proc/self/fd/1 /lib/udev/devices/stdout
    ln -sv /proc/self/fd/2 /lib/udev/devices/stderr
    ln -sv /proc/kcore /lib/udev/devices/core

    before_date=`date -R`

    #编译
    make EXTRAS="`echo extras/*/`" || eval "echo \"error: make error in udev\" && exit 1"
    make DESTDIR=/ EXTRAS="`echo extras/*/`" install
    cp -v etc/udev/rules.d/[0-9]* /etc/udev/rules.d/
    cd udev-config-6.3
    make install
    make install-doc
    make install-extra-doc
    cd ..
    install -m644 -v docs/writing_udev_rules/index.html \
	/usr/share/doc/udev-113/index.html


    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R udev-113
}

build_util_linux()
{
    #检测
    cd /sources

    check_file util-linux-2.12r.tar.bz2
    check_file util-linux-2.12r-cramfs-1.patch
    check_file util-linux-2.12r-lseek-1.patch
    check_dir util-linux-2.12r

    #解压
    tar jxvf util-linux-2.12r.tar.bz2
    cd util-linux-2.12r

    #特殊
    sed -e 's@etc/adjtime@var/lib/hwclock/adjtime@g' \
	-i $(grep -rl '/etc/adjtime' .)
    mkdir -pv /var/lib/hwclock

    #打补丁
    patch -Np1 -i ../util-linux-2.12r-cramfs-1.patch
    patch -Np1 -i ../util-linux-2.12r-lseek-1.patch

    before_date=`date -R`

    #编译
    ./configure
    make HAVE_KILL=yes HAVE_SLN=yes || eval "echo \"error: make error in util_linux\" && exit 1"
    make HAVE_KILL=yes HAVE_SLN=yes install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd /sources/
    rm -f -R util-linux-2.12r
}

build_vim()
{
    #检测
    cd /sources

    check_file vim-7.1.tar.bz2
    check_file vim-7.1-fixes-1.patch
    check_file vim-7.1-mandir-1.patch
    check_dir vim71

    #解压
    tar jxvf vim-7.1.tar.bz2
    cd vim71

    #打补丁
    patch -Np1 -i ../vim-7.1-fixes-1.patch
    patch -Np1 -i ../vim-7.1-mandir-1.patch

    #特殊
    echo '#define SYS_VIMRC_FILE "/etc/vimrc"' >> src/feature.h

    before_date=`date -R`

    #编译
    ./configure --prefix=/usr --enable-multibyte || eval "echo \"error: make error in vim\" && exit 1"
    make
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    ln -sv vim /usr/bin/vi
    for L in "" fr it pl ru; do
	ln -sv vim.1 /usr/share/man/$L/man1/vi.1
    done
    ln -sv ../vim/vim71/doc /usr/share/doc/vim-7.1
    cat > /etc/vimrc << "EOF"
" Begin /etc/vimrc

set nocompatible
set backspace=2
syntax on
if (&term == "iterm") || (&term == "putty")
  set background=dark
endif

" End /etc/vimrc
EOF

    #清理
    cd /sources/
    rm -f -R vim71
}

logout_comment()
{
    echo "****************"
    echo "run the fllowing command then logout"
    echo
    echo "# passwd root"
    echo "# logout"
    echo
    echo "****************"
}

before_date_sum=`date -R`

build_bzip2
build_diffutils
build_file
build_findutils
build_flex
build_grub
build_gawk
build_gettext
build_grep
build_groff
build_gzip
build_inetutils
build_iproute
build_kbd
build_less
build_make
build_man_db
build_mktemp
build_module_init_tools
build_patch
build_psmisc
build_shadow
build_sysklogd
build_sysvinit
build_tar
build_texinfo
build_udev
build_util_linux
build_vim

#计时
after_date_sum=`date -R`
echo "$before_date_sum"
echo "$after_date_sum"

logout_comment
