#! /bin/sh

source lfs.conf
source lfs.lib

check_lfs

build_binutils()
{
    #检测
    cd $LFS/sources

    check_file binutils-2.17.tar.bz2
    check_dir binutils-2.17
    check_dir binutils-build

    #解压
    tar jxvf binutils-2.17.tar.bz2
    cd binutils-2.17

    before_date=`date -R`

    #编译
    mkdir -v ../binutils-build
    cd ../binutils-build
    CC="gcc -B/usr/bin/" ../binutils-2.17/configure \
	--prefix=/tools --disable-nls --disable-werror
    make || eval "echo \"error: make error in binutils\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    make -C ld clean
    make -C ld LIB_PATH=/tools/lib
    cp -v ld/ld-new /tools/bin

    #清理
    cd $LFS/sources/
    rm -f -R binutils-2.17
    rm -f -R binutils-build
}

build_gcc()
{
    #检测
    cd $LFS/sources

    check_file gcc-4.1.2.tar.bz2
    check_dir gcc-4.1.2
    check_dir gcc-build

    #解压
    tar jxvf gcc-4.1.2.tar.bz2
    cd gcc-4.1.2

    before_date=`date -R`

    #编译
    mkdir -v ../gcc-build
    cd ../gcc-build
    CC="gcc -B/usr/bin/" ../gcc-4.1.2/configure --prefix=/tools \
	--with-local-prefix=/tools --disable-nls --enable-shared \
	--enable-languages=c
    make bootstrap || eval "echo \"error: make error in gcc\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R gcc-4.1.2
    rm -f -R gcc-build

    #特殊
    ln -vs gcc /tools/bin/cc
}

build_linux_api_headers()
{
    #检测
    cd $LFS/sources

    check_file linux-2.6.22.5.tar.bz2
    check_dir linux-2.6.22.5

    #解压
    tar jxvf linux-2.6.22.5.tar.bz2
    cd linux-2.6.22.5

    before_date=`date -R`

    #编译
    make mrproper
    make headers_check
    make INSTALL_HDR_PATH=dest headers_install
    cp -rv dest/include/* /tools/include

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R linux-2.6.22.5
}

build_glibc()
{
    #检测
    cd $LFS/sources

    check_file glibc-2.5.1.tar.bz2
    check_dir glibc-2.5.1
    check_dir glibc-build

    #解压
    tar jxvf glibc-2.5.1.tar.bz2
    cd glibc-2.5.1

    before_date=`date -R`

    #编译
    mkdir -v ../glibc-build
    cd ../glibc-build

    ../glibc-2.5.1/configure --prefix=/tools \
	--disable-profile --enable-add-ons \
	--enable-kernel=2.6.0 --with-binutils=/tools/bin \
	--without-gd --with-headers=/tools/include \
	--without-selinux
    make || eval "echo \"error: make error in glibc\" && exit 1"
    mkdir -pv /tools/etc
    touch /tools/etc/ld.so.conf
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R glibc-2.5.1
    rm -f -R glibc-build
}

adjust_toolchain()
{
    mv -v /tools/bin/{ld,ld-old}
    mv -v /tools/$(gcc -dumpmachine)/bin/{ld,ld-old}
    mv -v /tools/bin/{ld-new,ld}
    ln -sv /tools/bin/ld /tools/$(gcc -dumpmachine)/bin/ld

    gcc -dumpspecs | sed 's@^/lib/ld-linux.so.2@/tools&@g' \
	> `dirname $(gcc -print-libgcc-file-name)`/specs

    GCC_INCLUDEDIR=`dirname $(gcc -print-libgcc-file-name)`/include &&
    find ${GCC_INCLUDEDIR}/* -maxdepth 0 -xtype d -exec rm -rvf '{}' \; &&
    rm -vf `grep -l "DO NOT EDIT THIS FILE" ${GCC_INCLUDEDIR}/*` &&
    unset GCC_INCLUDEDIR

    echo 'main(){}' > dummy.c
    cc dummy.c
    readelf -l a.out | grep ': /tools'
    #read input
    rm -v dummy.c a.out
}

build_tcl()
{
    #检测
    cd $LFS/sources

    check_file tcl8.4.15-src.tar.gz
    check_dir tcl8.4.15

    #解压
    tar zxvf tcl8.4.15-src.tar.gz
    cd tcl8.4.15

    before_date=`date -R`

    #编译
    cd unix
    ./configure --prefix=/tools
    make || eval "echo \"error: make error in tcl\" && exit 1"
    make install && make install-private-headers

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    ln -sv tclsh8.4 /tools/bin/tclsh

    #清理
    cd $LFS/sources/
    rm -f -R tcl8.4.15
}

build_expect()
{
    #检测
    cd $LFS/sources

    check_file expect-5.43.0.tar.gz
    check_file expect-5.43.0-spawn-1.patch
    check_dir expect-5.43

    #解压
    tar zxvf expect-5.43.0.tar.gz
    cd expect-5.43

    #特殊
    patch -Np1 -i ../expect-5.43.0-spawn-1.patch
    cp configure{,.bak}
    sed 's:/usr/local/bin:/bin:' configure.bak > configure

    before_date=`date -R`

    #编译
    ./configure --prefix=/tools --with-tcl=/tools/lib \
	--with-tclinclude=/tools/include --with-x=no
    make || eval "echo \"error: make error in expect\" && exit 1"
    make SCRIPTS="" install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R expect-5.43
}

build_dejagnu()
{
    #检测
    cd $LFS/sources

    check_file dejagnu-1.4.4.tar.gz
    check_dir dejagnu-1.4.4

    #解压
    tar zxvf dejagnu-1.4.4.tar.gz
    cd dejagnu-1.4.4

    before_date=`date -R`

    #编译
    ./configure --prefix=/tools && \
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R dejagnu-1.4.4
}

build_gcc_pass2()
{
    #测试
    expect -c "spawn ls"
    #read input

    #检测
    cd $LFS/sources

    check_file gcc-4.1.2.tar.bz2
    check_file gcc-4.1.2-specs-1.patch
    check_dir gcc-4.1.2
    check_dir gcc-build

    #解压
    tar jxvf gcc-4.1.2.tar.bz2
    cd gcc-4.1.2

    #特殊
    cp -v gcc/Makefile.in{,.orig}
    sed 's@\./fixinc\.sh@-c true@' gcc/Makefile.in.orig > gcc/Makefile.in
    cp -v gcc/Makefile.in{,.tmp}
    sed 's/^XCFLAGS =$/& -fomit-frame-pointer/' gcc/Makefile.in.tmp \
	> gcc/Makefile.in
    patch -Np1 -i ../gcc-4.1.2-specs-1.patch

    before_date=`date -R`

    #编译
    mkdir -v ../gcc-build
    cd ../gcc-build
    ../gcc-4.1.2/configure --prefix=/tools \
	--with-local-prefix=/tools --enable-clocale=gnu \
	--enable-shared --enable-threads=posix \
	--enable-__cxa_atexit --enable-languages=c,c++ \
	--disable-libstdcxx-pch
    make || eval "echo \"error: make error in gcc pass2\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R gcc-4.1.2
    rm -f -R gcc-build

    #测试
    echo 'main(){}' > dummy.c
    cc dummy.c
    readelf -l a.out | grep ': /tools'
    #read input
    rm -v dummy.c a.out
}

build_binutils_pass2()
{
    #检测
    cd $LFS/sources

    check_file binutils-2.17.tar.bz2
    check_dir binutils-2.17
    check_dir binutils-build

    #解压
    tar jxvf binutils-2.17.tar.bz2
    cd binutils-2.17

    before_date=`date -R`

    #编译
    mkdir -v ../binutils-build
    cd ../binutils-build
    ../binutils-2.17/configure --prefix=/tools \
	--disable-nls --with-lib-path=/tools/lib
    make || eval "echo \"error: make error in binutils pass2\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    make -C ld clean
    make -C ld LIB_PATH=/usr/lib:/lib
    cp -v ld/ld-new /tools/bin

    #清理
    cd $LFS/sources/
    rm -f -R binutils-2.17
    rm -f -R binutils-build
}

build_ncurses()
{
    #检测
    cd $LFS/sources

    check_file ncurses-5.6.tar.gz
    check_dir ncurses-5.6

    #解压
    tar zxvf ncurses-5.6.tar.gz
    cd ncurses-5.6

    before_date=`date -R`

    #编译
    ./configure --prefix=/tools --with-shared \
	--without-debug --without-ada --enable-overwrite
    make || eval "echo \"error: make error in ncurses\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R ncurses-5.6
}

build_bash()
{
    #检测
    cd $LFS/sources

    check_file bash-3.2.tar.gz
    check_file bash-3.2-fixes-5.patch
    check_dir bash-3.2

    #解压
    tar zxvf bash-3.2.tar.gz
    cd bash-3.2

    #打补丁
    patch -Np1 -i ../bash-3.2-fixes-5.patch

    before_date=`date -R`

    #编译
    ./configure --prefix=/tools --without-bash-malloc
    make || eval "echo \"error: make error in bash\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    ln -vs bash /tools/bin/sh

    #清理
    cd $LFS/sources/
    rm -f -R bash-3.2
}

build_bzip2()
{
    #检测
    cd $LFS/sources

    check_file bzip2-1.0.4.tar.gz
    check_dir bzip2-1.0.4

    #解压
    tar zxvf bzip2-1.0.4.tar.gz
    cd bzip2-1.0.4

    before_date=`date -R`

    #编译
    make || eval "echo \"error: make error in bzip2\" && exit 1"
    make PREFIX=/tools install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R bzip2-1.0.4
}

build_coreutils()
{
    #检测
    cd $LFS/sources

    check_file coreutils-6.9.tar.bz2
    check_dir coreutils-6.9

    #解压
    tar jxvf coreutils-6.9.tar.bz2
    cd coreutils-6.9

    before_date=`date -R`

    #编译
    ./configure --prefix=/tools
    make || eval "echo \"error: make error in coreutils\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #特殊
    cp -v src/su /tools/bin/su-tools

    #清理
    cd $LFS/sources/
    rm -f -R coreutils-6.9
}

build_diffutils()
{
    #检测
    cd $LFS/sources

    check_file diffutils-2.8.1.tar.gz
    check_dir diffutils-2.8.1

    #解压
    tar zxvf diffutils-2.8.1.tar.gz
    cd diffutils-2.8.1

    before_date=`date -R`

    #编译
    ./configure --prefix=/tools
    make || eval "echo \"error: make error in diffutils\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R diffutils-2.8.1
}

build_findutils()
{
    #检测
    cd $LFS/sources

    check_file findutils-4.2.31.tar.gz
    check_dir findutils-4.2.31

    #解压
    tar zxvf findutils-4.2.31.tar.gz
    cd findutils-4.2.31

    before_date=`date -R`

    #编译
    ./configure --prefix=/tools
    make || eval "echo \"error: make error in findutils\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R findutils-4.2.31
}

build_gawk()
{
    #检测
    cd $LFS/sources

    check_file gawk-3.1.5.tar.bz2
    check_dir gawk-3.1.5

    #解压
    tar jxvf gawk-3.1.5.tar.bz2
    cd gawk-3.1.5

    before_date=`date -R`

    #编译
    ./configure --prefix=/tools
    cat >> config.h << "EOF"
#define HAVE_LANGINFO_CODESET 1
#define HAVE_LC_MESSAGES 1
EOF
    make || eval "echo \"error: make error in gawk\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R gawk-3.1.5
}

build_gettext()
{
    #检测
    cd $LFS/sources

    check_file gettext-0.16.1.tar.gz
    check_dir gettext-0.16.1

    #解压
    tar zxvf gettext-0.16.1.tar.gz
    cd gettext-0.16.1

    before_date=`date -R`

    #编译
    cd gettext-tools
    ./configure --prefix=/tools --disable-shared
    make -C gnulib-lib
    make -C src msgfmt
    cp -v src/msgfmt /tools/bin

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R gettext-0.16.1
}

build_grep()
{
    #检测
    cd $LFS/sources

    check_file grep-2.5.1a.tar.bz2
    check_dir grep-2.5.1a

    #解压
    tar jxvf grep-2.5.1a.tar.bz2
    cd grep-2.5.1a

    before_date=`date -R`

    #编译
    ./configure --prefix=/tools \
	--disable-perl-regexp
    make || eval "echo \"error: make error in grep\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R grep-2.5.1a
}

build_gzip()
{
    #检测
    cd $LFS/sources

    check_file gzip-1.3.12.tar.gz
    check_dir gzip-1.3.12

    #解压
    tar zxvf gzip-1.3.12.tar.gz
    cd gzip-1.3.12

    before_date=`date -R`

    #编译
    ./configure --prefix=/tools
    make || eval "echo \"error: make error in gzip\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R gzip-1.3.12
}

build_make()
{
    #检测
    cd $LFS/sources

    check_file make-3.81.tar.bz2
    check_dir make-3.81

    #解压
    tar jxvf make-3.81.tar.bz2
    cd make-3.81

    before_date=`date -R`

    #编译
    ./configure --prefix=/tools
    make || eval "echo \"error: make error in make\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R make-3.81
}

build_patch()
{
    #检测
    cd $LFS/sources

    check_file patch-2.5.4.tar.gz
    check_dir patch-2.5.4

    #解压
    tar zxvf patch-2.5.4.tar.gz
    cd patch-2.5.4

    before_date=`date -R`

    #编译
    ./configure --prefix=/tools
    make || eval "echo \"error: make error in patch\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R patch-2.5.4
}

build_perl()
{
    #检测
    cd $LFS/sources

    check_file perl-5.8.8.tar.bz2
    check_file perl-5.8.8-libc-2.patch
    check_dir perl-5.8.8

    #解压
    tar jxvf perl-5.8.8.tar.bz2
    cd perl-5.8.8

    #打补丁
    patch -Np1 -i ../perl-5.8.8-libc-2.patch

    before_date=`date -R`

    #编译
    ./configure.gnu --prefix=/tools -Dstatic_ext='Data/Dumper Fcntl IO POSIX'
    make perl utilities || eval "echo \"error: make error in perl\" && exit 1"
    cp -v perl pod/pod2man /tools/bin
    mkdir -pv /tools/lib/perl5/5.8.8
    cp -Rv lib/* /tools/lib/perl5/5.8.8

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R perl-5.8.8
}

build_sed()
{
    #检测
    cd $LFS/sources

    check_file sed-4.1.5.tar.gz
    check_dir sed-4.1.5

    #解压
    tar zxvf sed-4.1.5.tar.gz
    cd sed-4.1.5

    before_date=`date -R`

    #编译
    ./configure --prefix=/tools
    make || eval "echo \"error: make error in sed\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R sed-4.1.5
}

build_tar()
{
    #检测
    cd $LFS/sources

    check_file tar-1.18.tar.bz2
    check_dir tar-1.18

    #解压
    tar jxvf tar-1.18.tar.bz2
    cd tar-1.18

    before_date=`date -R`

    #编译
    ./configure --prefix=/tools
    make || eval "echo \"error: make error in tar\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R tar-1.18
}

build_texinfo()
{
    #检测
    cd $LFS/sources

    check_file texinfo-4.9.tar.bz2
    check_dir texinfo-4.9

    #解压
    tar jxvf texinfo-4.9.tar.bz2
    cd texinfo-4.9

    before_date=`date -R`

    #编译
    ./configure --prefix=/tools
    make || eval "echo \"error: make error in texinfo\" && exit 1"
    make install

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R texinfo-4.9
}

build_util_linux()
{
    #检测
    cd $LFS/sources

    check_file util-linux-2.12r.tar.bz2
    check_dir util-linux-2.12r

    #解压
    tar jxvf util-linux-2.12r.tar.bz2
    cd util-linux-2.12r

    #特殊
    sed -i 's@/usr/include@/tools/include@g' configure

    before_date=`date -R`

    #编译
    ./configure
    make -C lib
    make -C mount mount umount
    make -C text-utils more
    cp -v mount/{,u}mount text-utils/more /tools/bin

    #计时
    after_date=`date -R`
    echo "$before_date"
    echo "$after_date"

    #清理
    cd $LFS/sources/
    rm -f -R util-linux-2.12r
}

stripping()
{
    strip --strip-debug /tools/lib/*
    strip --strip-unneeded /tools/{,s}bin/*
    rm -rf /tools/{info,man}
}

logout_comment()
{
    echo "****************"
    echo "now exit from user lfs and type commonds below"
    echo
    echo "$ exit"
    echo
    echo "# export LFS=$LFS"
    echo "# chown -R root:root \$LFS/tools"
    echo
    echo "you can backup $LFS/tools for reuse"
    echo "****************"
}

if [ ! -e $LFS/sources ] ; then
    echo "error: $LFS/sources don't exist"
    exit 1
fi

before_date_sum=`date -R`

build_binutils
build_gcc
build_linux_api_headers
build_glibc
adjust_toolchain
build_tcl
build_expect
build_dejagnu
build_gcc_pass2
build_binutils_pass2
build_ncurses
build_bash
build_bzip2
build_coreutils
build_diffutils
build_findutils
build_gawk
build_gettext
build_grep
build_gzip
build_make
build_patch
build_perl
build_sed
build_tar
build_texinfo
build_util_linux
stripping

#计时
after_date_sum=`date -R`
echo "$before_date_sum"
echo "$after_date_sum"

logout_comment
